# -*- coding: utf-8 -*-

import argparse

from lintaosp.__version__ import __version__


class Argument(object):
    def __init__(self):
        self._parser = argparse.ArgumentParser(description="Lint AOSP")
        self._add()

    def _add(self):
        self._parser.add_argument(
            "--config-file",
            action="store",
            dest="config_file",
            help="config file (.yml)",
            required=True,
        )
        group = self._parser.add_mutually_exclusive_group()
        group.add_argument(
            "--lint-project",
            action="store",
            default="",
            dest="lint_project",
            help="lint project (/path/to/project)",
            required=False,
        )
        group.add_argument(
            "--listen-url",
            action="store",
            default="",
            dest="listen_url",
            help="listen url (host:port)",
            required=False,
        )
        self._parser.add_argument(
            "--output-file",
            action="store",
            default="",
            dest="output_file",
            help="output file (.json|.txt|.xlsx)",
            required=False,
        )
        self._parser.add_argument(
            "-v", "--version", action="version", version=__version__
        )

    def parse(self, argv):
        return self._parser.parse_args(argv[1:])



# -*- coding: utf-8 -*-

import queue
import threading

QUEUE_NUM = 1
ROUTINE_NUM = 1


class QueueException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Queue(object):
    def __init__(self, config):
        if config is None:
            pass
        self._queue = queue.Queue(QUEUE_NUM)
        for _ in range(ROUTINE_NUM):
            Worker(self._queue)

    def _add(self, routine, args):
        self._queue.put((routine, args))

    def _wait(self):
        self._queue.join()

    def run(self, routine, args=None):
        for _ in range(QUEUE_NUM):
            self._add(routine, args)
        self._wait()


class WorkerException(QueueException):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Worker(threading.Thread):
    def __init__(self, _queue):
        super(Worker, self).__init__()
        self._queue = _queue
        self.daemon = True
        self.start()

    def run(self):
        while True:
            routine, args = self._queue.get()
            routine(args)
            self._queue.task_done()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: flow.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="flow.proto",
    package="flow",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\nflow.proto\x12\x04\x66low"\x1e\n\x0b\x46lowRequest\x12\x0f\n\x07message\x18\x01 \x01(\t"\x1c\n\tFlowReply\x12\x0f\n\x07message\x18\x01 \x01(\t2=\n\tFlowProto\x12\x30\n\x08SendFlow\x12\x11.flow.FlowRequest\x1a\x0f.flow.FlowReply"\x00\x62\x06proto3',
)


_FLOWREQUEST = _descriptor.Descriptor(
    name="FlowRequest",
    full_name="flow.FlowRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="message",
            full_name="flow.FlowRequest.message",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=20,
    serialized_end=50,
)


_FLOWREPLY = _descriptor.Descriptor(
    name="FlowReply",
    full_name="flow.FlowReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="message",
            full_name="flow.FlowReply.message",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=52,
    serialized_end=80,
)

DESCRIPTOR.message_types_by_name["FlowRequest"] = _FLOWREQUEST
DESCRIPTOR.message_types_by_name["FlowReply"] = _FLOWREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FlowRequest = _reflection.GeneratedProtocolMessageType(
    "FlowRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _FLOWREQUEST,
        "__module__": "flow_pb2"
        # @@protoc_insertion_point(class_scope:flow.FlowRequest)
    },
)
_sym_db.RegisterMessage(FlowRequest)

FlowReply = _reflection.GeneratedProtocolMessageType(
    "FlowReply",
    (_message.Message,),
    {
        "DESCRIPTOR": _FLOWREPLY,
        "__module__": "flow_pb2"
        # @@protoc_insertion_point(class_scope:flow.FlowReply)
    },
)
_sym_db.RegisterMessage(FlowReply)


_FLOWPROTO = _descriptor.ServiceDescriptor(
    name="FlowProto",
    full_name="flow.FlowProto",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=82,
    serialized_end=143,
    methods=[
        _descriptor.MethodDescriptor(
            name="SendFlow",
            full_name="flow.FlowProto.SendFlow",
            index=0,
            containing_service=None,
            input_type=_FLOWREQUEST,
            output_type=_FLOWREPLY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_FLOWPROTO)

DESCRIPTOR.services_by_name["FlowProto"] = _FLOWPROTO

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

import abc


class AospAbstract(abc.ABC):
    def __init__(self, config):
        self._config = config

    @abc.abstractmethod
    def _execution(self, project):
        pass

    def run(self, project):
        return self._execution(project)



# -*- coding: utf-8 -*-


"""Prototype
{
  "file": "name",
  "line": 1,
  "type": "Error",
  "details": "text"
}
"""


class Format:
    DETAILS = "details"
    FILE = "file"
    LINE = "line"
    TYPE = "type"


class Type:
    ERROR = "Error"
    INFO = "Info"
    WARN = "Warning"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import setuptools

about = {}
here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'lintaosp', '__version__.py'), 'r') as f:
    exec(f.read(), about)

with open('README.md', 'r') as f:
    readme = f.read()

with open('requirements.txt') as f:
    requirements = [l for l in f.read().splitlines() if l]

setuptools.setup(
    author=about['__author__'],
    author_email=about['__author_email__'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    description=about['__description__'],
    download_url='https://github.com/craftslab/lintaosp/archive/v%s.tar.gz' % about['__version__'],
    entry_points={'console_scripts': ['lintaosp=lintaosp.main:main']},
    include_package_data=True,
    install_requires=requirements,
    keywords=['lint', 'aosp'],
    license=about['__license__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    name=about['__title__'],
    packages=setuptools.find_packages(exclude=['examples', 'ez_setup', 'release', 'script', 'tests', 'tests.*']),
    package_data={'lintaosp': ['config/*.yml']},
    url=about['__url__'],
    version=about['__version__'],
    zip_safe=False)



# -*- coding: utf-8 -*-

import os

from lintaosp.aosp.sdk import Sdk
from lintaosp.config.config import ConfigFile
from lintaosp.printer.printer import Printer


class AospException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Aosp(object):
    def __init__(self, config):
        if config is None:
            raise AospException("config invalid")
        self._config = config
        self._spec = config.config_file.get(ConfigFile.SPEC, None)
        if self._spec is None:
            raise AospException("spec invalid")
        self._instance = self._instantiate()

    def _dump(self, data):
        printer = Printer()
        printer.run(data=data, name=self._config.output_file, append=False)

    def _instantiate(self):
        buf = {}
        if Sdk.__name__.lower() in self._spec:
            buf[Sdk.__name__.lower()] = Sdk(self._spec[Sdk.__name__.lower()])
        return buf

    def routine(self, project):
        if not isinstance(project, str) or not os.path.exists(project):
            raise AospException("project invalid")
        buf = {}
        for key in self._spec.keys():
            if key in self._instance.keys():
                buf[key] = self._instance[key].run(project)
        if len(self._config.output_file) != 0:
            self._dump(buf)
        return buf



# -*- coding: utf-8 -*-

import json
import openpyxl
import os
import time

from lintaosp.proto.proto import Format
from openpyxl.styles import Alignment, Font

head = {
    "A": Format.FILE,
    "B": Format.LINE,
    "C": Format.TYPE,
    "D": Format.DETAILS,
}


class PrinterException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Printer(object):
    _format = [".json", ".txt", ".xlsx"]

    def __init__(self, config=None):
        if config is None:
            pass

    @staticmethod
    def format():
        return Printer._format

    def _json(self, data, name):
        with open(name, "w", encoding="utf-8") as f:
            f.write(json.dumps(data, ensure_ascii=False, indent=2))

    def _txt(self, data, name):
        def _txt_helper(data, out):
            global head
            for key in sorted(head.keys()):
                out.write(u"%s: %s\n" % (head[key], data[head[key]]))

        with open(name, "w", encoding="utf8") as f:
            f.write("")
            for key, val in data.items():
                for v in val:
                    f.write(u"%s\n" % key)
                    _txt_helper(v, f)
                    f.write("\n")

    def _xlsx(self, data, name):
        def _styling_head(sheet):
            for item in head.keys():
                sheet[item + "1"].alignment = Alignment(
                    horizontal="center", shrink_to_fit=True, vertical="center"
                )
                sheet[item + "1"].font = Font(bold=True, name="Calibri")
            sheet.freeze_panes = sheet["A2"]

        def _styling_data(sheet, rows):
            for key in head.keys():
                for row in range(rows):
                    sheet[key + str(row + 2)].alignment = Alignment(
                        horizontal="center", vertical="center"
                    )
                    sheet[key + str(row + 2)].font = Font(bold=False, name="Calibri")

        wb = openpyxl.Workbook()
        wb.remove(wb.active)
        for key, val in data.items():
            ws = wb.create_sheet()
            ws.append([head[key].upper() for key in sorted(head.keys())])
            ws.title = "%s %s" % (
                key.upper(),
                time.strftime("%Y-%m-%d", time.localtime(time.time())),
            )
            for v in val:
                ws.append([v[head[key]] for key in sorted(head.keys())])
            _styling_head(ws)
            _styling_data(ws, len(val))
        wb.save(filename=name)

    def run(self, data, name, append=True):
        if append is True:
            raise PrinterException("append not supported")
        func = Printer.__dict__.get(os.path.splitext(name)[1].replace(".", "_"), None)
        if func is not None:
            func(self, data, name)
